@@ -2,7 +2,7 @@
  * cwdaemon - morse sounding daemon for the parallel or serial port
  * Copyright (C) 2002 - 2005 Joop Stakenborg <pg4i@amsat.org>
  *		        and many authors, see the AUTHORS file.
- * Copyright (C) 2012 - 2014 Kamil Ignacak <acerion@wp.pl>
+ * Copyright (C) 2012 - 2015 Kamil Ignacak <acerion@wp.pl>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -347,7 +347,14 @@ static unsigned char ptt_flag = 0;
 /* Don't turn PTT off until cwdaemon sends back an echo to client.
    client may request echoing back to it a reply when cwdaemon finishes
    playing given request. PTT shouldn't be turned off when sending the
-   reply (TODO: why it shouldn't?). */
+   reply (TODO: why it shouldn't?).
+
+   This flag is set whenever client sends request for sending back a
+   reply (i.e. either <ESC>h request, or caret request).
+
+   This flag is re-set whenever such reply is sent (to be more
+   precise: after playing a requested text, but just before sending to
+   the client the requested reply). */
 #define PTT_ACTIVE_ECHO		0x04
 
 
@@ -747,7 +754,7 @@ void cwdaemon_set_ptt_on(cwdevice *device, const char *info)
 #endif
 
 		ptt_flag |= PTT_ACTIVE_AUTO;
-		cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag +PTT_ACTIVE_AUTO (%02d, %d)", ptt_flag, __LINE__);
+		cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag +PTT_ACTIVE_AUTO (0x%02x/%s)", ptt_flag, cwdaemon_debug_ptt_flags());
 	}
 
 	return;
@@ -767,7 +774,7 @@ void cwdaemon_set_ptt_off(cwdevice *device, const char *info)
 {
 	device->ptt(device, OFF);
 	ptt_flag = 0;
-	cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag = 0 (%02d, %d)", ptt_flag, __LINE__);
+	cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag = 0 (0x%02x/%s)", ptt_flag, cwdaemon_debug_ptt_flags());
 
 	cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, info);
 
@@ -786,6 +793,8 @@ void cwdaemon_set_ptt_off(cwdevice *device, const char *info)
    Parameter type is uint32_t, which gives us maximum of 4294967295
    seconds, i.e. ~136 years. Should be enough.
 
+   TODO: change the argument type to size_t.
+
    \param seconds - time of tuning
 */
 void cwdaemon_tune(uint32_t seconds)
@@ -811,6 +820,12 @@ void cwdaemon_tune(uint32_t seconds)
 
 /**
    \brief Reset some initial parameters of cwdaemon and libcw
+
+   TODO: split this function into:
+   cwdaemon_reset_basic_params()
+   cwdaemon_reset_libcw_output()
+   and call these two functions separately instead of this one.
+   This function that combines these two doesn't make much sense.
 */
 void cwdaemon_reset_almost_all(void)
 {
@@ -913,7 +928,13 @@ void cwdaemon_reset_libcw_output(void)
 {
 	/* This function is called when cwdaemon receives '0' escape code.
 	   README describes this code as "Reset to default values".
-	   Therefore we use default_* below. */
+	   Therefore we use default_* below.
+
+	   However, the function is called after "current_" values
+	   have been reset to "default_" values. So maybe we could use
+	   "current_" values and somehow encapsulate the calls to
+	   cw_set_*() functions? The calls are also made elsewhere.
+	*/
 
 	/* Delete old generator (if it exists). */
 	cwdaemon_close_libcw_output();
@@ -1019,9 +1040,10 @@ void cwdaemon_prepare_reply(char *reply, const char *request, size_t n)
 
 	   It is important to set this flag at the beginning of the function. */
 	ptt_flag |= PTT_ACTIVE_ECHO;
-	cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag +PTT_ACTIVE_ECHO (%02d, %d)", ptt_flag, __LINE__);
+	cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag +PTT_ACTIVE_ECHO (0x%02x/%s)", ptt_flag, cwdaemon_debug_ptt_flags());
 
-	memcpy(&reply_addr, &request_addr, sizeof(reply_addr)); /* Remember sender. */
+	/* We are sending reply to the same host that sent a request. */
+	memcpy(&reply_addr, &request_addr, sizeof(reply_addr));
 	reply_addrlen = request_addrlen;
 
 	strncpy(reply, request, n);
@@ -1116,7 +1138,7 @@ int cwdaemon_recvfrom(char *request, int n)
 	}
 
 	/* Remove CRLF if present. TCP buffer may end with '\n', so make
-	   sure that every request is consistently ended with NUL.
+	   sure that every request is consistently ended with NUL only.
 	   Do it early, do it now. */
 	int z;
 	while (recv_rc > 0
@@ -1138,6 +1160,8 @@ int cwdaemon_recvfrom(char *request, int n)
    Watch the socket and if there is an escape character check what it is,
    otherwise play morse.
 
+   FIXME: duplicate return value (zero and zero).
+
    \return 0 when an escape code has been received
    \return 0 when no request or an empty request has been received
    \return 1 when a text request has been played
@@ -1166,9 +1190,16 @@ int cwdaemon_receive(void)
 
 	request_buffer[recv_rc] = '\0';
 
+	cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "-------------------");
+	/* TODO: replace the magic number 27 with constant. */
 	if (request_buffer[0] != 27) {
 		/* No ESCAPE. All received data should be treated
-		   as text to be sent using Morse code. */
+		   as text to be sent using Morse code.
+
+		   Note that this does not exclude possibility of
+		   caret request (e.g. "some text^"), which does
+		   require sending a reply to client. Such request is
+		   correctly handled by cwdaemon_play_request(). */
 		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "request: \"%s\"", request_buffer);
 		if ((strlen(request_buffer) + strlen(request_queue)) <= CWDAEMON_REQUEST_QUEUE_SIZE_MAX - 1) {
 			strcat(request_queue, request_buffer);
@@ -1178,6 +1209,14 @@ int cwdaemon_receive(void)
 		}
 		return 1;
 	} else {
+		/* Don't print literal escape value, use <ESC>
+		   symbol. First reason is that the literal value
+		   doesn't look good in console (some non-printable
+		   glyph), second reason is that printing <ESC>c to
+		   terminal makes funny things with the lines already
+		   printed to the terminal (tested in xfce4-terminal
+		   and xterm). */
+		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "escaped request: \"<ESC>%s\"", request_buffer + 1);
 		cwdaemon_handle_escaped_request(request_buffer);
 		return 0;
 	}
@@ -1207,9 +1246,8 @@ void cwdaemon_handle_escaped_request(char *request)
 		global_cwdevice->reset(global_cwdevice);
 
 		ptt_flag = 0;
-		cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag = 0 (%02d, %d)", ptt_flag, __LINE__);
-		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__,
-			       "resetting completed");
+		cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag = 0 (0x%02x/%s)", ptt_flag, cwdaemon_debug_ptt_flags());
+		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "resetting completed");
 
 		break;
 	case '2':
@@ -1257,7 +1295,7 @@ void cwdaemon_handle_escaped_request(char *request)
 				cwdaemon_set_ptt_off(global_cwdevice, "PTT off");
 			}
 			ptt_flag &= 0;
-			cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag = 0 (%02d, %d)", ptt_flag, __LINE__);
+			cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag = 0 (0x%02x/%s)", ptt_flag, cwdaemon_debug_ptt_flags());
 		}
 		break;
 	case '5':
@@ -1337,6 +1375,7 @@ void cwdaemon_handle_escaped_request(char *request)
 		break;
 	case 'c':
 		{
+			/* FIXME: change this uint32_t to size_t. */
 			uint32_t seconds = 0;
 			/* Tune for a number of seconds. */
 			if (cwdaemon_params_tune(&seconds, request + 2)) {
@@ -1354,7 +1393,7 @@ void cwdaemon_handle_escaped_request(char *request)
 			if (rv == 0) {
 				/* Value totally invalid. */
 				cwdaemon_debug(CWDAEMON_VERBOSITY_E, __func__, __LINE__,
-					       "invalid requested PTT delay [ms]: \"%s\" (should be between %d and %d inclusive)",
+					       "invalid requested PTT delay [ms]: \"%s\" (should be integer between %d and %d inclusive)",
 					       request + 2,
 					       CWDAEMON_PTT_DELAY_MIN, CWDAEMON_PTT_DELAY_MAX);
 			} else if (rv == 1) {
@@ -1541,9 +1580,9 @@ void cwdaemon_play_request(char *request)
 		case '^':
 			/* Send echo to main program when CW playing is done. */
 			*x = '\0';     /* Remove '^' and possible trailing garbage. */
-			/* I'm guessing here that '^' can be found at
-			   the end of request, and it means "echo text of current
-			   request back to sender once you finish playing it". */
+			/* '^' can be found at the end of request, and
+			   it means "echo text of current request back
+			   to client once you finish playing it". */
 			cwdaemon_prepare_reply(reply_buffer, request, strlen(request));
 
 			/* cwdaemon will wait for queue-empty callback
@@ -1560,6 +1599,7 @@ void cwdaemon_play_request(char *request)
 			cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "Morse character \"%c\" to be queued in libcw", *x);
 			cw_send_character(*x);
 			cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "Morse character \"%c\" has been queued in libcw", *x);
+
 			x++;
 			if (cw_get_gap() == 2) {
 				if (*x == '^') {
@@ -1644,7 +1684,7 @@ void cwdaemon_keyingevent(__attribute__((unused)) void *arg, int keystate)
 void cwdaemon_tone_queue_low_callback(__attribute__((unused)) void *arg)
 {
 	int len = cw_get_tone_queue_length();
-	cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: start, TQ len = %d, PTT flag = %02x/%s",
+	cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: start, TQ len = %d, PTT flag = 0x%02x/%s",
 		       len, ptt_flag, cwdaemon_debug_ptt_flags());
 
 	if (len > tq_low_watermark) {
@@ -1667,7 +1707,7 @@ void cwdaemon_tone_queue_low_callback(__attribute__((unused)) void *arg)
 		   Feel free to correct me ;) */
 
 
-		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: branch 1, PTT flag = %02d/%s", ptt_flag, cwdaemon_debug_ptt_flags());
+		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: branch 1, PTT flag = 0x%02x/%s", ptt_flag, cwdaemon_debug_ptt_flags());
 
 		cwdaemon_set_ptt_off(global_cwdevice, "PTT (auto) off");
 
@@ -1677,14 +1717,14 @@ void cwdaemon_tone_queue_low_callback(__attribute__((unused)) void *arg)
 		   the server (i.e. cwdaemon) after the server plays
 		   all characters. */
 
-		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: branch 2, PTT flag = %02d/%s", ptt_flag, cwdaemon_debug_ptt_flags());
+		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: branch 2, PTT flag = 0x%02x/%s", ptt_flag, cwdaemon_debug_ptt_flags());
 
 		/* Since echo is being sent, we can turn the flag off.
 		   For some reason cwdaemon works better when we turn the
 		   flag off before sending the reply, rather than turning
 		   if after sending the reply. */
 		ptt_flag &= ~PTT_ACTIVE_ECHO;
-		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: PTT flag -PTT_ACTIVE_ECHO, PTT flag = %02d/%s", ptt_flag, cwdaemon_debug_ptt_flags());
+		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: PTT flag -PTT_ACTIVE_ECHO, PTT flag = 0x%02x/%s", ptt_flag, cwdaemon_debug_ptt_flags());
 
 
 		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: echoing \"%s\" back to client             <----------", reply_buffer);
@@ -1724,10 +1764,10 @@ void cwdaemon_tone_queue_low_callback(__attribute__((unused)) void *arg)
 	} else {
 		/* TODO: how to correctly handle this case?
 		   Should we do something? */
-		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: branch 3, PTT flag = %02d/%s", ptt_flag, cwdaemon_debug_ptt_flags());
+		cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: branch 3, PTT flag = 0x%02x/%s", ptt_flag, cwdaemon_debug_ptt_flags());
 	}
 
-	cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: end, TQ len = %d, PTT flag = %02x/%s\n\n",
+	cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "low TQ callback: end, TQ len = %d, PTT flag = 0x%02x/%s",
 		       cw_get_tone_queue_length(), ptt_flag, cwdaemon_debug_ptt_flags());
 
 	return;
@@ -1809,7 +1849,7 @@ void cwdaemon_args_process_long(int argc, char *argv[])
 					   strict, and accept only
 					   fully valid optarg. */
 					cwdaemon_debug(CWDAEMON_VERBOSITY_E, __func__, __LINE__,
-						       "invalid requested PTT delay [ms]: \"%s\" (should be between %d and %d inclusive)",
+						       "invalid requested PTT delay [ms]: \"%s\" (should be integer between %d and %d inclusive)",
 						       optarg,
 						       CWDAEMON_PTT_DELAY_MIN, CWDAEMON_PTT_DELAY_MAX);
 					exit(EXIT_FAILURE);
@@ -1906,7 +1946,7 @@ void cwdaemon_args_process_short(int c, const char *optarg)
 			   are very strict, and accept only fully
 			   valid optarg. */
 			cwdaemon_debug(CWDAEMON_VERBOSITY_E, __func__, __LINE__,
-				       "invalid requested PTT delay [ms]: \"%s\" (should be between %d and %d inclusive)",
+				       "invalid requested PTT delay [ms]: \"%s\" (should be integer between %d and %d inclusive)",
 				       optarg,
 				       CWDAEMON_PTT_DELAY_MIN, CWDAEMON_PTT_DELAY_MAX);
 			exit(EXIT_FAILURE);
@@ -1964,7 +2004,7 @@ bool cwdaemon_params_cwdevice(const char *optarg)
 	cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__,
 		       "requested cwdevice \"%s\"", optarg);
 
-	if (!cwdaemon_cwdevice_set(&global_cwdevice, optarg) == -1) {
+	if (!cwdaemon_cwdevice_set(&global_cwdevice, optarg)) {
 		return false;
 	} else {
 		return true;
@@ -2003,7 +2043,7 @@ bool cwdaemon_params_priority(int *priority, const char *optarg)
 	long lv = 0;
 	if (!cwdaemon_get_long(optarg, &lv) || lv < -20 || lv > 20) {
 		cwdaemon_debug(CWDAEMON_VERBOSITY_E, __func__, __LINE__,
-			       "invalid requested process priority: \"%s\" (should be between -20 and 20 inclusive)",
+			       "invalid requested process priority: \"%s\" (should be integer between -20 and 20 inclusive)",
 			       optarg);
 		return false;
 	} else {
@@ -2020,7 +2060,7 @@ bool cwdaemon_params_wpm(int *wpm, const char *optarg)
 	long lv = 0;
 	if (!cwdaemon_get_long(optarg, &lv) || lv < CW_SPEED_MIN || lv > CW_SPEED_MAX) {
 		cwdaemon_debug(CWDAEMON_VERBOSITY_E, __func__, __LINE__ ,
-			       "invalid requested morse speed [wpm]: \"%s\" (should be between %d and %d inclusive)",
+			       "invalid requested morse speed [wpm]: \"%s\" (should be integer between %d and %d inclusive)",
 			       optarg, CW_SPEED_MIN, CW_SPEED_MAX);
 		return false;
 	} else {
@@ -2039,7 +2079,7 @@ bool cwdaemon_params_tune(uint32_t *seconds, const char *optarg)
 	/* TODO: replace cwdaemon_get_long() with cwdaemon_get_uint32() */
 	if (!cwdaemon_get_long(optarg, &lv) || lv < 0 || lv > CWDAEMON_TUNE_SECONDS_MAX) {
 		cwdaemon_debug(CWDAEMON_VERBOSITY_E, __func__, __LINE__ ,
-			       "invalid requested tuning time [s]: \"%s\" (should be between %d and %d inclusive)",
+			       "invalid requested tuning time [s]: \"%s\" (should be integer between %d and %d inclusive)",
 			       optarg, 0, CWDAEMON_TUNE_SECONDS_MAX);
 		return false;
 	} else {
@@ -2092,7 +2132,7 @@ int cwdaemon_params_pttdelay(int *delay, const char *optarg)
 	long lv = 0;
 	if (!cwdaemon_get_long(optarg, &lv)) {
 		cwdaemon_debug(CWDAEMON_VERBOSITY_E, __func__, __LINE__,
-			       "invalid requested PTT delay [ms]: \"%s\" (should be integer value between %d and %d inclusive)",
+			       "invalid requested PTT delay [ms]: \"%s\" (should be integer between %d and %d inclusive)",
 			       optarg,
 			       CWDAEMON_PTT_DELAY_MIN, CWDAEMON_PTT_DELAY_MAX);
 
@@ -2141,7 +2181,7 @@ bool cwdaemon_params_volume(int *volume, const char *optarg)
 	long lv = 0;
 	if (!cwdaemon_get_long(optarg, &lv) || lv < CW_VOLUME_MIN || lv > CW_VOLUME_MAX) {
 		cwdaemon_debug(CWDAEMON_VERBOSITY_E, __func__, __LINE__,
-			       "invalid requested volume [%%]: \"%s\" (should be between %d and %d inclusive)",
+			       "invalid requested volume [%%]: \"%s\" (should be integer between %d and %d inclusive)",
 			       optarg, CW_VOLUME_MIN, CW_VOLUME_MAX);
 		return false;
 	} else {
@@ -2165,7 +2205,7 @@ bool cwdaemon_params_weighting(int *weighting, const char *optarg)
 	long lv = 0;
 	if (!cwdaemon_get_long(optarg, &lv) || lv < CWDAEMON_MORSE_WEIGHTING_MIN || lv > CWDAEMON_MORSE_WEIGHTING_MAX) {
 		cwdaemon_debug(CWDAEMON_VERBOSITY_E, __func__, __LINE__,
-			       "invalid requested weighting: \"%s\" (should be between %d and %d inclusive)",
+			       "invalid requested weighting: \"%s\" (should be integer between %d and %d inclusive)",
 			       optarg, CWDAEMON_MORSE_WEIGHTING_MIN, CWDAEMON_MORSE_WEIGHTING_MAX);
 		return false;
 	} else {
@@ -2182,7 +2222,7 @@ bool cwdaemon_params_tone(int *tone, const char *optarg)
 	long lv = 0;
 	if (!cwdaemon_get_long(optarg, &lv) || lv < CW_FREQUENCY_MIN || lv > CW_FREQUENCY_MAX) {
 		cwdaemon_debug(CWDAEMON_VERBOSITY_E, __func__, __LINE__,
-			       "invalid requested tone [Hz]: \"%s\" (should be between %d and %d inclusive)",
+			       "invalid requested tone [Hz]: \"%s\" (should be integer between %d and %d inclusive)",
 			       optarg, CW_FREQUENCY_MIN, CW_FREQUENCY_MAX);
 		return false;
 	} else {
@@ -2323,12 +2363,12 @@ bool cwdaemon_params_ptt_on_off(const char *optarg)
 		}
 
 		ptt_flag |= PTT_ACTIVE_MANUAL;
-		cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag +PTT_ACTIVE_MANUAL (%02d, %d)", ptt_flag, __LINE__);
+		cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag +PTT_ACTIVE_MANUAL (0x%02x/%s)", ptt_flag, cwdaemon_debug_ptt_flags());
 
 	} else if (ptt_flag & PTT_ACTIVE_MANUAL) {	/* only if manually activated */
 
 		ptt_flag &= ~PTT_ACTIVE_MANUAL;
-		cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag -PTT_ACTIVE_MANUAL (%02d, %d)", ptt_flag, __LINE__);
+		cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag -PTT_ACTIVE_MANUAL (0x%02x/%s)", ptt_flag, cwdaemon_debug_ptt_flags());
 
 		if (!(ptt_flag & !PTT_ACTIVE_AUTO)) {	/* no PTT modifiers */
 
@@ -2339,7 +2379,7 @@ bool cwdaemon_params_ptt_on_off(const char *optarg)
 			} else {
 				/* still sending, cannot yet switch PTT off */
 				ptt_flag |= PTT_ACTIVE_AUTO;	/* ensure auto-PTT active */
-				cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag +PTT_ACTIVE_AUTO (%02d, %d)", ptt_flag, __LINE__);
+				cwdaemon_debug(CWDAEMON_VERBOSITY_D, __func__, __LINE__, "PTT flag +PTT_ACTIVE_AUTO (0x%02x/%s)", ptt_flag, cwdaemon_debug_ptt_flags());
 
 				cwdaemon_debug(CWDAEMON_VERBOSITY_I, __func__, __LINE__, "reverting from PTT (manual) to PTT (auto) now");
 			}
@@ -2753,7 +2793,7 @@ bool cwdaemon_cwdevices_init(void)
 #elif defined HAVE_DEV_PPBUS_PPI_H    /* FreeBSD (ppbus/ppi). */
 	cwdevice_lp.desc = strdup("ppi0");
 #else                                 /* FIXME: where is OpenBSD? */
-	cwdevice_lp.desc = NULL;
+
 #endif
 
 
@@ -2800,10 +2840,12 @@ void cwdaemon_cwdevices_free(void)
 		cwdevice_null.desc = NULL;
 	}
 
+#if defined (HAVE_LINUX_PPDEV_H) || defined (HAVE_DEV_PPBUS_PPI_H)
 	if (cwdevice_lp.desc) {
 		free(cwdevice_lp.desc);
 		cwdevice_lp.desc = NULL;
 	}
+#endif
 
 	return;
 }
